﻿

----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBarsLayout")

L:RegisterTranslations("enUS", function() return {
	["Update"] = true,
	["Update everything."] = true,

	["Reset Position"] = true,
	["Move BunchOfBars to the middle of the screen."] = true,
	
	["Scale"] = true,
	["The scale of the frames."] = true,

	["Show in party"] = true,
	["Show BunchOrBars when your in a party."] = true
} end)

L:RegisterTranslations("koKR", function() return {
	["Update"] = "업데이트",
	["Update everything."] = "모두 업데이트합니다.",

	["Reset Position"] = "위치 초기화",
	["Move BunchOfBars to the middle of the screen."] = "모든 프레임을 중앙에 오도록 초기화 합니다.",
	
	["Scale"] = "크기",
	["The scale of the frames."] = "프레임의 크기를 설정합니다.",

	["Show in party"] = "파티 보기",
	["Show BunchOrBars when your in a party."] = "파티시에도 표시합니다."
} end)

----------------------------------
--      Local Declaration      --
----------------------------------

local DogTag          = AceLibrary("DogTag-1.0")
local SharedMedia     = AceLibrary("SharedMedia-1.0")
local SharedMediaType = SharedMedia.MediaType and SharedMedia.MediaType.STATUSBAR or "statusbar"

local moduleorder

local sorts = {
	["NAME"]  = "Name",
	["INDEX"] = "Index"
}
local orders = {
	["CLASS"] = "Class",
	["GROUP"] = "Group",
	["NONE"]  = "None"
}



----------------------------------
--      Module Declaration      --
----------------------------------

BunchOfBars.options.args.visual.args = {
	update = {
		type = "execute",
		name = L["Update"],
		desc = L["Update everything."],
		func = "UpdateAll"
	},
	reset = {
		type = "execute",
		name = L["Reset Position"],
		desc = L["Move BunchOfBars to the middle of the screen."],
		func = "ResetPosition",
		disabled = function() return InCombatLockdown() end
	},
	scale = {
		type = "range",
		name = L["Scale"],
		desc = L["The scale of the frames."],
		min  = 0.5,
		max  = 2,
		step = 0.1,
		get  = "GetSetScale",
		set  = "GetSetScale",
		disabled = function() return InCombatLockdown() end
	},
	party = {
		type = "toggle",
		name = L["Show in party"],
		desc = L["Show BunchOrBars when your in a party."],
		get  = "GetSetParty",
		set  = "GetSetParty",
		disabled = function() return InCombatLockdown() end
	},

	--[[sort = {
		type  = "text",
		name  = "Sort by",
		desc  = "Sort the groups by ...",
		usage = "",
		validate = sorts,
		get = "GetSetSort",
		set = "GetSetSort",
		disabled = function() return InCombatLockdown() end
	},
	order = {
		type  = "text",
		name  = "Order by",
		desc  = "Order groups by ...",
		usage = "",
		validate = orders,
		get = "GetSetOrder",
		set = "GetSetOrder",
		disabled = function() return InCombatLockdown() end
	},
	orderstr = {
		type  = "text",
		name  = "Order String",
		desc  = "How to order the groups",
		usage = "...",
		get   = "GetSetOrderStr",
		set   = "GetSetOrderStr",
		disabled = function() return InCombatLockdown() end
	}]]--
}


BunchOfBars.defaults.visual = {
	scale = 1.0,
	locked = false,
	padding = 5,
	x = 400,
	y = 400,
	party = true,
	sort = "NAME",
	order = "CLASS",
	orderstr = "WARRIOR,DRUID,PALADIN,PRIEST,SHAMAN,ROGUE,MAGE,WARLOCK,HUNTER"
}



----------------------------------
--      Module Functions        --
----------------------------------

function BunchOfBars:InitializeLayout()
	ClickCastFrames = ClickCastFrames or {}

	self.frames = {}

	SharedMedia:Register(SharedMediaType, "Charcoal", "Interface\\Addons\\BunchOfBars\\Textures\\Charcoal")
end


function BunchOfBars:OnProfileEnable() -- will be called when our profile is reset
	self:UpdateAll()
end


function BunchOfBars:OnEnable(first)
	if not first then return end

	--[[
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
	--]]

	if IsInGuild() then
		self:RegisterEvent("CHAT_MSG_ADDON")
		SendAddonMessage("BunchOfBars", self.revision, "GUILD")
	end

	self:RegisterModules()

	self:UpdateLayoutOrder()

	-- The point at which we create this frame seems important, if we do it to early some stuff doesn't work.
	self:ScheduleEvent(self.CreateMaster, 2, self)
end


function BunchOfBars:CreateMaster()
	local frame = CreateFrame("Frame", "BunchOfBarsFrame", UIParent, "SecureGroupHeaderTemplate")
	self.master = frame -- self.frame is used by FuBarPlugin

	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	
	frame:SetHeight(1)
	frame:SetWidth(1)

	self:PositionAndScale()


	--[[
	nameList = [STRING] -- a comma separated list of player names (not used if 'groupFilter' is set)
	groupFilter = [1-8, STRING] -- a comma seperated list of raid group numbers and/or uppercase class names
	strictFiltering = [BOOLEAN] - if true, then characters must match both a group and a class from the groupFilter list
	point = [STRING] -- a valid XML anchoring point (Default: "TOP")
	xOffset = [NUMBER] -- the x-Offset to use when anchoring the unit buttons (Default: 0)
	yOffset = [NUMBER] -- the y-Offset to use when anchoring the unit buttons (Default: 0)
	sortMethod = ["INDEX", "NAME"] -- defines how the group is sorted (Default: "INDEX")
	sortDir = ["ASC", "DESC"] -- defines the sort order (Default: "ASC")
	template = [STRING] -- the XML template to use for the unit buttons
	templateType = [STRING] - specifies the frame type of the managed subframes (Default: "Button")
	columnSpacing
	columnAnchorPoint
	initial-width
	initial-height
	initial-scale
	initial-unitWatch has to do with RegisterUnitWatch
	groupBy = ["GROUP", "CLASS"]
	groupingOrder = [1-8, STRING] -- a comma seperated list of raid group numbers and/or uppercase class names
	maxColumns
	unitsPerColumn
	--]]
	frame:SetAttribute("showRaid", true)
	frame:SetAttribute("showParty", self.db.profile.visual.party)
	frame:SetAttribute("showPlayer", true)
	--frame:SetAttribute("showSolo", true) -- Useful for testing.

	frame:SetAttribute("yOffset", -1) -- space between frames

	--frame:SetAttribute("sortMethod", self.db.profile.visual.sort)
	--frame:SetAttribute("groupBy", self.db.profile.visual.group)
	--frame:SetAttribute("groupingOrder", self.db.profile.visual.orderstr)
	frame:SetAttribute("sortMethod", "NAME")
	frame:SetAttribute("groupBy", "CLASS")
	frame:SetAttribute("groupingOrder", "WARRIOR,DRUID,PALADIN,PRIEST,SHAMAN,ROGUE,MAGE,WARLOCK,HUNTER")

	frame.initialConfigFunction = self.ConfigureFrame
	frame:SetAttribute("template", "SecureUnitButtonTemplate")
	
	frame:Show() -- direct after this show it will start creating unit frames
end


function BunchOfBars:PositionAndScale()
	self.master:SetScale(self.db.profile.visual.scale)

	local s = self.master:GetEffectiveScale()
	local x, y = self.db.profile.visual.x / s, self.db.profile.visual.y / s

	self.master:ClearAllPoints()
	self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end


local function OnAttributeChanged(frame, name, value)
	if name ~= "unit" then return end -- we are only interrested in the unit attribute

	if frame.unit and not value and frame.created then
		BunchOfBars:ModuleOnInactive(frame)

		BunchOfBars.frames[frame.unit] = nil
		frame.unit = nil
	elseif value then
		if frame.unit then BunchOfBars.frames[frame.unit] = nil end
		frame.unit = value
		BunchOfBars.frames[frame.unit] = frame

		BunchOfBars:ModuleOnUpdate(frame)
		BunchOfBars.master:StopMovingOrSizing() -- seems to fix a bug where you can't stop dragging
	end
end


local function OnDragStart()
	if not BunchOfBars.db.profile.visual.locked and not InCombatLockdown() then
		BunchOfBars.master:StartMoving()
	end
end

local function OnDragStop()
	BunchOfBars.master:StopMovingOrSizing()

	local x,y = BunchOfBars.master:GetLeft(), BunchOfBars.master:GetTop()
	local s   = BunchOfBars.master:GetEffectiveScale()

	BunchOfBars.db.profile.visual.x = x*s
	BunchOfBars.db.profile.visual.y = y*s
end


local function OnEnter()
	DogTag:OnMouseoverUpdate()
end

local function OnLeave()
	DogTag:OnMouseoverUpdate()
end


function BunchOfBars.ConfigureFrame(frame)
	frame:SetAttribute("*type1", "target")

	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:RegisterForDrag("LeftButton")

	frame:SetScript("OnDragStart", OnDragStart)
	frame:SetScript("OnDragStop", OnDragStop)
	frame:SetScript("OnEnter", OnEnter)
	frame:SetScript("OnLeave", OnLeave)
	frame:SetScript("OnAttributeChanged", OnAttributeChanged)

	frame:SetAlpha(1.0)

	BunchOfBars:ModuleOnCreate(frame)
	BunchOfBars:UpdateLayout(frame) -- apperantly we are able to change the size of the frame here, even in combat

	ClickCastFrames[frame] = true
end


function BunchOfBars:ModuleOnCreate(frame)
	frame.parts = {}

	for name,module in self:IterateModules() do
		frame.parts[name] = module:OnCreate(frame)
	end
end

function BunchOfBars:ModuleOnUpdate(frame)
	for name,module in self:IterateModules() do
		module:OnUpdate(frame, frame.parts[name])
	end
end

function BunchOfBars:ModuleOnInactive(frame)
	for name,module in self:IterateModules() do
		module:OnInactive(frame, frame.parts[name])
	end
end


function BunchOfBars:UpdateLayoutOrder()
	moduleorder = {}

	for name,module in self:IterateModules() do
		if module.db.profile.position then
			local done = false
			for k,v in ipairs(moduleorder) do
				if module.db.profile.position < self:GetModule(v).db.profile.position then
					table.insert(moduleorder, k, name)
					done = true
					break
				end
			end

			if not done then
				table.insert(moduleorder, name)
			end
		end
	end
end


function BunchOfBars:UpdateLayout(frame)
	local last = nil
	local width = 0

	-- Order and show them
	for _,name in ipairs(moduleorder) do
		local p = frame.parts[name]

		width = width + frame.parts[name]:GetWidth() + self.db.profile.visual.padding

		p:ClearAllPoints()
		p:SetPoint("TOP", frame, "TOP", 0, (p:GetHeight() - 20) / 4)

		if not last then
			p:SetPoint("LEFT", frame, "LEFT", self.db.profile.visual.padding, 0)
		else
			p:SetPoint("LEFT", last, "RIGHT", self.db.profile.visual.padding, 0)
		end

		last = p
	end

	frame:SetWidth(math.ceil(width) + 1)
	frame:SetHeight(20)
end

function BunchOfBars:UpdateLayouts()
	for _,frame in pairs(self.frames) do
		self:UpdateLayout(frame)
	end
end


function BunchOfBars:UpdateAll()
	for _,frame in pairs(self.frames) do
		self:ModuleOnUpdate(frame)
		self:UpdateLayout(frame)
	end
end


function BunchOfBars:ResetPosition()
	self.master:ClearAllPoints()
	self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
end




BunchOfBars.modulePrototype.core = BunchOfBars
BunchOfBars.modulePrototype.defaultDB = {}


function BunchOfBars.modulePrototype:OnCreate(frame)
	return nil -- do nothing
end

function BunchOfBars.modulePrototype:OnUpdate(frame, mypart)
	-- do nothing
end

function BunchOfBars.modulePrototype:OnInactive(frame, mypart)
	-- do nothing
end


function BunchOfBars.modulePrototype:UpdateAll()
	for _,frame in pairs(self.core.frames) do
		self:OnUpdate(frame, frame.parts[self.name])
	end
end





function BunchOfBars:GetSetPadding(value)
	if type(value) == "nil" then return self.db.profile.visual.padding end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.visual.padding ~= value then
		self.db.profile.visual.padding = value
		
		self:UpdateLayouts()
	end
end



function BunchOfBars:GetSetScale(value)
	if type(value) == "nil" then return self.db.profile.visual.scale end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.visual.scale ~= value then
		self.db.profile.visual.scale = value
		
		self:PositionAndScale()
	end
end


function BunchOfBars:GetSetParty(v)
	if type(v) == "nil" then return self.db.profile.visual.party end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.visual.party ~= v then
		self.db.profile.visual.party = v
		
		self.master:SetAttribute("showParty", self.db.profile.visual.party)
	end
end


function BunchOfBars:GetSetSort(v)	
	if type(v) == "nil" then return self.db.profile.visual.sort end

	if self.db.profile.visual.sort ~= v then
		self.db.profile.visual.sort = v

		self:Print(self.db.profile.visual.sort)
		self.master:SetAttribute("sortMethod", self.db.profile.visual.sort)
	end
end


function BunchOfBars:GetSetOrder(v)
	if type(v) == "nil" then return self.db.profile.visual.order end

	if self.db.profile.visual.order ~= v then
		self.db.profile.visual.order = v

		self:Print(self.db.profile.visual.group)
		self:Print(self.db.profile.visual.orderstr)
		self.master:SetAttribute("groupBy", self.db.profile.visual.group)
		self.master:SetAttribute("groupingOrder", self.db.profile.visual.orderstr)
	end
end


function BunchOfBars:GetSetOrderStr(v)
	if type(v) == "nil" then return self.db.profile.visual.orderstr end

	if self.db.profile.visual.orderstr ~= v then
		self.db.profile.visual.orderstr = v

		self:Print(self.db.profile.visual.group)
		self:Print(self.db.profile.visual.orderstr)
		self.master:SetAttribute("groupBy", self.db.profile.visual.group)
		self.master:SetAttribute("groupingOrder", self.db.profile.visual.orderstr)
	end
end
